//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Descriptor/SelectionVector.h
//! @brief     Defines class SelectionVector.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_DESCRIPTOR_SELECTIONVECTOR_H
#define BORNAGAIN_GUI_MODEL_DESCRIPTOR_SELECTIONVECTOR_H

#include "GUI/Model/Descriptor/SelectionProperty.h"

//! A special kind of owning vector, providing "standard" interfaces for
//! 'std::vector<SelectionProperty<Catalog>>', used in classes that work not just with single
//! SelectionProperty but with the set of them, for example 'InstrumentsSet', 'CompoundItem',
//! 'ParticleLayoutItems', 'MaskItems'

template <typename Catalog> class SelectionVector {
public:
    using CatalogedType = typename Catalog::CatalogedType;

    void delete_element(const CatalogedType* element)
    {
        for (size_t i = 0; i < m_selections.size(); i++)
            if (m_selections[i].certainItem() == element)
                m_selections.erase(m_selections.begin() + i);
    }

    void delete_at(size_t i) { m_selections.erase(m_selections.begin() + i); }

    void insert_at(size_t i, CatalogedType* element)
    {
        SelectionProperty<Catalog> newSelection;
        newSelection.setCertainItem(element);
        m_selections.insert(m_selections.begin() + i, std::move(newSelection));
    }

    int index_of(const CatalogedType* element) const
    {
        for (size_t i = 0; i < m_selections.size(); i++)
            if (m_selections[i].certainItem() == element)
                return int(i);
        return -1;
    }

    void move(size_t fromIndex, size_t toIndex)
    {
        if (fromIndex > toIndex)
            std::rotate(m_selections.rend() - fromIndex - 1, m_selections.rend() - fromIndex,
                        m_selections.rend() - toIndex);
        else
            std::rotate(m_selections.begin() + fromIndex, m_selections.begin() + fromIndex + 1,
                        m_selections.begin() + toIndex + 1);
    }

    QVector<CatalogedType*> toModifiableQVector() const
    {
        QVector<CatalogedType*> result;
        for (const auto& sel : m_selections)
            result.append(sel.certainItem());
        return result;
    }

    QVector<const CatalogedType*> toQVector() const
    {
        QVector<const CatalogedType*> result;
        for (const auto& sel : m_selections)
            result.append(sel.certainItem());
        return result;
    }

    void emplace_back(CatalogedType* item) { insert_at(m_selections.size(), item); }
    void clear() { m_selections.clear(); }
    size_t size() const { return m_selections.size(); }
    bool empty() const { return m_selections.empty(); }
    SelectionProperty<Catalog>& operator[](int i) { return m_selections[i]; }
    SelectionProperty<Catalog>& at(int i) { return m_selections.at(i); }
    SelectionProperty<Catalog>& front() { return m_selections.front(); }
    SelectionProperty<Catalog>& back() { return m_selections.back(); }

    using Iterator = typename std::vector<SelectionProperty<Catalog>>::iterator;
    using ConstIterator = typename std::vector<SelectionProperty<Catalog>>::const_iterator;

    ConstIterator begin() const { return m_selections.cbegin(); }
    ConstIterator end() const { return m_selections.cend(); }
    Iterator begin() { return m_selections.begin(); }
    Iterator end() { return m_selections.end(); }

private:
    std::vector<SelectionProperty<Catalog>> m_selections;
};

#endif // BORNAGAIN_GUI_MODEL_DESCRIPTOR_SELECTIONVECTOR_H
